#!/usr/bin/env bash

##--------------------------------------------------------------------
## Copyright (c) 2025 Dianomic Systems Inc.
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##--------------------------------------------------------------------

##
## Author: Ashish Jabble
##

#!/bin/bash
set -euo pipefail

log() {
    echo "[$(date +'%Y-%m-%d %H:%M:%S')] $*"
}

ARCH_RAW=$(arch)
case "$ARCH_RAW" in
    x86_64) ARCH="amd64"; ODBC_LINUX_DIR="linux" ;;
    aarch64) ARCH="aarch64"; ODBC_LINUX_DIR="linuxaarch64" ;;
    *) log "Unsupported architecture: $ARCH_RAW"; exit 1 ;;
esac

SNOWFLAKE_VERSION="3.8.0"
log "Installing Snowflake ODBC driver for architecture: ${ARCH}..."

SNOWFLAKE_PACKAGE_NAME="snowflake-odbc-${SNOWFLAKE_VERSION}.${ARCH}.deb"
SNOWFLAKE_PACKAGE_URL="https://sfc-repo.snowflakecomputing.com/odbc/${ODBC_LINUX_DIR}/${SNOWFLAKE_VERSION}/${SNOWFLAKE_PACKAGE_NAME}"

if ! dpkg -s "snowflake-odbc" >/dev/null 2>&1; then
    wget -q -P /tmp/ "$SNOWFLAKE_PACKAGE_URL"
    sudo apt install -y "/tmp/${SNOWFLAKE_PACKAGE_NAME}"
    rm -f "/tmp/${SNOWFLAKE_PACKAGE_NAME}"
else
    log "Snowflake ODBC driver already installed."
fi

sudo apt install -y unixodbc unixodbc-dev odbcinst1debian2

log "Snowflake ODBC installation completed."

log "Installing Databricks ODBC driver for architecture: ${ARCH}..."
VERSION="2.9.1"
SIMBASPARK_ZIP="SimbaSparkODBC-${VERSION}.1001-Debian-64bit.zip"

if [ "$ARCH" = "amd64" ]; then
    PACKAGE_NAME="simbaspark_${VERSION}.1001-2_amd64.deb"
else
    log "Databricks ODBC driver not available for architecture: $ARCH"
    exit 0
fi

PACKAGE_URL="https://databricks-bi-artifacts.s3.us-east-2.amazonaws.com/simbaspark-drivers/odbc/${VERSION}/${SIMBASPARK_ZIP}"

sudo apt install -y unzip libsasl2-modules-gssapi-mit

if ! dpkg -s "simbaspark" >/dev/null 2>&1; then
    wget -q -P /tmp/ "$PACKAGE_URL"
    unzip -o "/tmp/${SIMBASPARK_ZIP}" -d /tmp/
    sudo apt install -y "/tmp/${PACKAGE_NAME}"
    rm -f "/tmp/${SIMBASPARK_ZIP}" "/tmp/${PACKAGE_NAME}"
else
    log "Databricks ODBC driver already installed."
fi

if ! grep -q "\[DatabricksODBCDriver\]" /etc/odbcinst.ini; then
    sudo tee -a /etc/odbcinst.ini > /dev/null <<EOF

[DatabricksODBCDriver]
APILevel=1
Description=Databricks ODBC Connector
Driver=/opt/simba/spark/lib/64/libsparkodbc_sb64.so
UsageCount=1
EOF
fi

log "Databricks ODBC installation completed."
